/**********************************************************************************************************************
   LEDMOTION.h
 **********************************************************************************************************************/
/**********************************************************************************************************************
  Copyright (c) 2017 Roel Arits

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
 **********************************************************************************************************************/

#ifndef LEDMOTION_H
#define LEDMOTION_H

/***********************************************************************************************************************
   I N C L U D E S
 **********************************************************************************************************************/
#include <Adafruit_NeoPixel.h>

/***********************************************************************************************************************
   E X P O R T E D   S Y M B O L   D E F I N I T I O N S (defines, typedefs)
 **********************************************************************************************************************/

#define MOTION_DEBUG_OFF         0
#define MOTION_DEBUG_ALL         1
#define MOTION_DEBUG_MOTION      2
#define MOTION_DEBUG_STATES      3
#define MOTION_DEBUG             MOTION_DEBUG_STATES

typedef enum
{
  DIRECTION_NONE, 
  DIRECTION_POSITIVE, 
  DIRECTION_NEGATIVE
} TDirection;

typedef enum
{
  MOTION_STATE_READY,
  MOTION_STATE_MOVING,
  MOTION_STATE_CHANGING_DIR,
  MOTION_STATE_DECELERATING,
  MOTION_STATE_DECEL_STOPPED,
  MOTION_STATE_STOPPED_AT_END,
  MOTION_STATE_STOPPED_AT_START,
} TMotionState;

typedef enum
{
  MOTION_EVENT_NONE,
  MOTION_EVENT_MOVE,
  MOTION_EVENT_CHANGE_DIR,
  MOTION_EVENT_STOP,
} TMotionEvent;

/***********************************************************************************************************************
   E X P O R T E D   D A T A
 **********************************************************************************************************************/

/***********************************************************************************************************************
   I N L I N E   F U N C T I O N S
 **********************************************************************************************************************/

/***********************************************************************************************************************
   E X P O R T E D   F U N C T I O N   P R O T O T Y P E S
 **********************************************************************************************************************/

/***********************************************************************************************************************
   CLASS
 **********************************************************************************************************************/
class TLEDMotion
{
  public:
    uint32_t startTime;

    uint8_t LEDNr;
    uint8_t startLEDNr;
    uint8_t prevLEDNr;

    TDirection dir;
    TDirection prevDir;
    TDirection decelerationDir;

    // Distance is always positive
    float distance;
    float prevDistance;
    float storedDistance;

    TMotionState state;
    TMotionState newState;

    TMotionEvent event;
    TMotionEvent prevEvent;

    // Velocity is always positive
    float velocity;
    float prevVelocity;
    float storedVelocity;

    float acceleration;
    float deceleration;
    uint32_t LEDColor;

  public:
    TLEDMotion ();
    ~TLEDMotion ();

    void Init (Adafruit_NeoPixel *pStrip, uint8_t argStartPos, uint8_t argEndPos, uint32_t argSpeed, uint32_t argColor);
    bool Move (Adafruit_NeoPixel *pStrip);
    void ConvertAngleToAccelearation (Adafruit_NeoPixel *pStrip, float argAngle);
    void UpdateDirection (float pitch);
    void CheckForDirectionChange (float pitch);
    uint32_t ConvertByteToColor (byte data, byte max_data);
    void Main (Adafruit_NeoPixel *pStrip, float yawn, float pitch, float roll);

    void PrepareStart(Adafruit_NeoPixel *pStrip);

  private:
    void HandleEvent (void);
    void HandleStateChange (void);

    void HandleStartMovingState(Adafruit_NeoPixel *pStrip, float elapsedTime);
    void HandleDeceleratingState(Adafruit_NeoPixel *pStrip, float elapsedTime);
    void HandleBouncingState(Adafruit_NeoPixel *pStrip, float elapsedTime);

    void MoveLEDPos (Adafruit_NeoPixel *pStrip);
    void MoveLEDNeg (Adafruit_NeoPixel *pStrip);

    void PrepareDeceleration(void);
};

#endif
